//	Application.cpp - Class handling the various application bits and pieces.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifdef DEBUG
#pragma warning(disable : 4786) //Stupid MS headers...
#endif

#include "Application.h"
#include "FatalError.h"
#include "ConfigLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/ScreenRes.h"
#include "NiallsDrawingLib/Drawer.h"
#include "Random.h"
#include "Fonts.h"
#include "FilePath.h"
#include "GameCounter.h"
#include "DebugStatement.h"
#include "MusicSingleton.h"
#include "Sounds.h"

#include "TitleScreen.h"
#include "GamePlay.h"
#include "GameOverScreen.h"
#include "PickupSelectScreen.h"

#include "SDL_image.h"
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <cmath>

using namespace std;

//------------------------------------------------------------------------------
Application::Application(const CmdLineParams &commandLine):
timeToQuit(false),
lastRepaint(0),
width(0),
height(0),
noSound(false),
noMusic(false),
offlineTime(0),
state(0),
prevLoudBit(false),
loudBit(false),
beat(false),
winter(false)
{
	int i;
	int aaLevel = 0;
	bool antialias = false;
	CmdLineParams::const_iterator it;
	stringstream tempConverter;
	bool fullscreen = true;
	bool editor = false;
	string audioAPI;

	DebugStatement("Application::Application()");

	//Initialise starting pickups array.
	for(i=0;i<(Pickup::NumTypes-1);++i)
		startPickups[i] = false;

	//Initialise key mappings.
	initKeyManager();

	//Initialise SDL.
	if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0)
	{
		ostringstream err("Could not initialise SDL: ");
		err << SDL_GetError();
		throw FatalError(err.str());
	}

	DebugStatement("Application: SDL Initialised");

	//Set window name.
	SDL_WM_SetCaption("Rainbow Space Donkey Escape", 0);

	DebugStatement("Application: Window name set");

	if(commandLine.size() > 0)
	{
		//Determine whether to print the help from the command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if((*it) == "--help")
			{
				cout << endl;
				cout << "Rainbow Space Donkey Escape";
				cout << endl << endl;
				cout << "Usage:" << endl;
#ifdef WIN32
				cout << "RainbowSpaceDonkeyEscape.exe -w [width] -h [height] -a [val] win -ns -nm -aa [name]";
#else
				cout << "RainbowSpaceDonkeyEscape -w [width] -h [height] -a [val] -win -ns -nm -aa [name]";
#endif
				cout << endl << endl;
				cout << "Flags:" << endl;
				cout << "--width [val] | -w [val]:\tSet the width of the window.";
				cout << endl;
				cout << "--height [val] | -h [val]:\tSet the height of the window.";
				cout << endl;
				cout << "--antialias [val] | -a [val]:\tSets the antialiasing level (2, 4, or 6).";
				cout << endl;
				cout << "--windowed | -win:\t\tSets whether to start the game in a window rather than fullscreen.";
				cout << endl;
				cout << "--nosound | -ns:\t\tSwitches off the sound output.";
				cout << endl;
				cout << "--nomusic | -nm:\t\tSwitches off the background music.";
				cout << endl;
				cout << "--audio-api [name] | -aa [name]:\t\tSets the audio API to use.  Can be DirectSound, ASIO, MME, ALSA, JACK, OSS, CoreAudio.";
				cout << endl;
				cout << "--help:\t\t\t\tDisplay this help.";
				cout << endl;
				throw FatalError("\n");
			}
		}

		//Determine width from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--width") || ((*it) == "-w"))
			{
				++it;
				if(it != commandLine.end())
				{
					tempConverter << *it;
					tempConverter >> width;
					tempConverter.clear();
				}
				else
					break;
			}
		}
		if(!width)
		{
			width = ScreenResolution::getScreenWidth();
			cout << "Using default width = " << width << endl;
		}

		//Determine height from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--height") || ((*it) == "-h"))
			{
				++it;
				if(it != commandLine.end())
				{
					tempConverter << *it;
					tempConverter >> height;
					tempConverter.clear();
				}
				else
					break;
			}
		}
		if(!height)
		{
			height = ScreenResolution::getScreenHeight();
			cout << "Using default height = " << height << endl;
		}

		//Determine whether to go fullscreen from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--windowed") || ((*it) == "-win"))
				fullscreen = false;
		}

		//Determine whether to switch off sound from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--nosound") || ((*it) == "-ns"))
				noSound = true;
		}

		//Determine whether to switch off music from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--nomusic") || ((*it) == "-nm"))
				noMusic = true;
		}

		//Determine which audio API to use.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--audio-api") || ((*it) == "-aa"))
			{
				++it;
				if(it != commandLine.end())
					audioAPI = *it;
				else
					break;
			}
		}

		//Determine whether to use antialiasing from command line.
		for(it=commandLine.begin();it!= commandLine.end();++it)
		{
			if(((*it) == "--antialias") || ((*it) == "-a"))
			{
				antialias = true;
				++it;
				if(it != commandLine.end())
				{
					tempConverter << *it;
					tempConverter >> aaLevel;
					tempConverter.clear();
				}
				else
					break;
			}
		}
		if(antialias && (aaLevel < 2))
		{
			aaLevel = 2;
			cout << "Using 2x antialiasing." << endl;
		}
	}
	else
	{
		DebugStatement("Application: Loading config settings from file.");

		width = config.getScreenWidth();
		height = config.getScreenHeight();
		if(config.getAntialiasLevel() > 0)
		{
			antialias = true;
			aaLevel = config.getAntialiasLevel();
		}
		fullscreen = config.getFullscreen();
		if(config.getNoSound())
			noSound = true;
		if(config.getNoMusic())
			noMusic = true;
		audioAPI = config.getAudioAPI();
	}
	winterUnlocked = config.getWinter();
	DebugStatement("Application: Config settings loaded.");

	//Create the drawer.
	drawer = new OpenGLDrawer(1280, 720);
	DebugStatement("Application: OpenGLDrawer created.");

	drawer->initialise(width, height, fullscreen, aaLevel);
	DebugStatement("Application: OpenGLDrawer initialised.");

	//Create the initial state.
	state = new TitleScreen(this);
	DebugStatement("Application: TitleScreen created.");

	if((offlineFilename == "") && (offlineTime == 0))
	{
		SDL_JoystickEventState(SDL_ENABLE);

		//Enable joysticks, if available.
		for(i=0;i<SDL_NumJoysticks();++i)
		{
			joysticks.push_back(SDL_JoystickOpen(i));
			joyUpIsDown.push_back(false);
			joyDownIsDown.push_back(false);
			joyLeftIsDown.push_back(false);
			joyRightIsDown.push_back(false);
		}

		//Start audio.
		portAudioDevice.setCallback(this);
		portAudioDevice.setNumChannels(0, 2);
		portAudioDevice.setSamplerate(44100.0);
		portAudioDevice.setAPI(audioAPI);
		if(!noSound)
		{
			MusicSingleton::getInstance().setApplication(this);
			MusicSingleton::getInstance().addTrack("RainbowSpaceDonkey.ogg");
			MusicSingleton::getInstance().playTrack("RainbowSpaceDonkey");
			Sounds::getInstance();//Load stuff.
			portAudioDevice.start();
			DebugStatement("Application: Audio started.");
		}
	}

	//Hide the cursor.
	SDL_ShowCursor(SDL_DISABLE);

	
	DebugStatement("Application::Application() end");
}

//------------------------------------------------------------------------------
Application::~Application()
{
	unsigned int i;

	if((offlineFilename == "") && (offlineTime == 0))
	{
		if(!noSound)
			portAudioDevice.stop();

		for(i=0;i<joysticks.size();++i)
			SDL_JoystickClose(joysticks[i]);
	}

	delete drawer;
	if(state)
		delete state;

	SDL_Quit();
}

//------------------------------------------------------------------------------
void Application::getAudio(float *input, float *output, int numSamples)
{
	int i;

	//Clear the buffers first.
	for(i=0;i<numSamples;++i)
	{
		output[i*2] = 0.0f;
		output[(i*2)+1] = 0.0f;
	}

	//Play the background music.
	if(!noMusic)
		MusicSingleton::getInstance().audioBlock(output, numSamples);

	//Play any Sounds.
	Sounds::getInstance().getAudio(output, numSamples);

	//Add any sound generated by state.
	//****BIG BUG HERE - THREADING ISSUE****
	/*if(state)
		state->getAudio(output, numSamples);*/
}

//------------------------------------------------------------------------------
void Application::eventLoop()
{
	Uint32 tempint;
	SDL_Event event;

	loadGlyphs();

	while(1)
	{
		//Handle events.
		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
				case SDL_KEYDOWN:
					keyDown(event.key.keysym);
					break;
				case SDL_KEYUP:
					keyUp(event.key.keysym);
					break;
				case SDL_MOUSEBUTTONDOWN:
					mouseDown(event.button.button,
							  event.button.x,
							  event.button.y);
					break;
				case SDL_MOUSEBUTTONUP:
					mouseUp(event.button.button,
							event.button.x,
							event.button.y);
					break;
				case SDL_MOUSEMOTION:
					mouseMove(event.motion.x, event.motion.y);
					break;
				case SDL_QUIT:
					timeToQuit = true;
					break;
				case SDL_JOYAXISMOTION:
					joyMove(event.jaxis.which,
							event.jaxis.axis,
							(float)(event.jaxis.value)/32768.0f);
					break;
				case SDL_JOYBUTTONDOWN:
					joyDown(event.jbutton.which, event.jbutton.button);
					break;
				case SDL_JOYBUTTONUP:
					joyUp(event.jbutton.which, event.jbutton.button);
					break;
			}
		}

		if(timeToQuit)
			break;

		if(beat)
		{
			state->triggerBeat();
			snowflakes.setBeat();
			beat = false;
		}
		if(loudBit != prevLoudBit)
		{
			prevLoudBit = loudBit;
			state->setLoudBit(loudBit);
		}

		//Repaint the screen, or wait a bit first.
		//We aim for a 60fps frame rate here.
		tempint = lastRepaint;
		lastRepaint = SDL_GetTicks();
		if((lastRepaint - tempint) < 15)
			SDL_Delay(15 - (lastRepaint - tempint));

		//Do the drawing.
		drawer->beginDrawing();
		if(state)
			state->draw(drawer);
		if(winter)
			snowflakes.draw(drawer);
		drawer->endDrawing();
	}
}

//------------------------------------------------------------------------------
void Application::quit()
{
	timeToQuit = true;
}

//------------------------------------------------------------------------------
void Application::nextState(const string newState, const string extra)
{
	DebugStatement("Application::nextState() newState = ");
	DebugStatement(newState);
	if(newState == "TitleScreen")
	{
		GameCounter::getInstance().reset();
		delete state;
		state = new TitleScreen(this);
		DebugStatement("Application::nextState() TitleScreen created.");
	}
	else if(newState == "GamePlay")
	{
		GameCounter::getInstance().reset();
		delete state;
		state = new GamePlay(this);
		DebugStatement("Application::nextState() GamePlay created.");
	}
	else if(newState == "GameOverScreen")
	{
		GameCounter::getInstance().reset();
		delete state;
		state = new GameOverScreen(this);
		DebugStatement("Application::nextState() GameOverScreen created.");
	}
	else if(newState == "PickupSelectScreen")
	{
		GameCounter::getInstance().reset();
		delete state;
		state = new PickupSelectScreen(this);
		DebugStatement("Application::nextState() PickupSelectScreen created.");
	}
	state->setLoudBit(loudBit);
}

//------------------------------------------------------------------------------
void Application::setLoudBit(bool val)
{
	loudBit = val;
}

//------------------------------------------------------------------------------
void Application::triggerBeat()
{
	beat = true;
}

//------------------------------------------------------------------------------
void Application::setWinter(bool val)
{
	winter = val;

	if(!winterUnlocked)
	{
		winterUnlocked = true;
		config.setWinter();
	}
}

//------------------------------------------------------------------------------
void Application::setStartPickups(bool *pickups)
{
	int i;

	for(i=0;i<(Pickup::NumTypes-1);++i)
		startPickups[i] = pickups[i];
}

//------------------------------------------------------------------------------
void Application::keyDown(const SDL_keysym& keyPressed)
{
	if(state)
		state->keyDown(keyManager.getAction(keyPressed), keyPressed);
}

//------------------------------------------------------------------------------
void Application::keyUp(const SDL_keysym& keyPressed)
{
	if(state)
		state->keyUp(keyManager.getAction(keyPressed), keyPressed);
}

//------------------------------------------------------------------------------
void Application::mouseDown(int button, int posX, int posY)
{
	float x, y;
	float offset;
	const float dW = drawer->getDrawWidth();
	const float dH = drawer->getDrawHeight();
	const float dAspect = dH/dW;
	const float aspect = (float)height/(float)width;

	x = (float)posX/(float)width;
	x *= dW;
	y = (float)posY/(float)height;
	y *= dH;

	if(aspect < dAspect)
	{
		offset = (float)width - (((float)height/dH) * dW);
		offset *= 0.5f;
	}
	else if(aspect > dAspect)
	{
		offset = (float)height - (((float)width/dW) * dH);
		offset *= 0.5f;
	}

	if(state)
	{
		if(button == SDL_BUTTON_WHEELUP)
			state->mouseWheel(false);
		else if(button == SDL_BUTTON_WHEELDOWN)
			state->mouseWheel(true);
		else
			state->mouseDown(button, (int)x, (int)y);
	}
}

//------------------------------------------------------------------------------
void Application::mouseUp(int button, int posX, int posY)
{
	float x, y;
	float offset;
	const float dW = drawer->getDrawWidth();
	const float dH = drawer->getDrawHeight();
	const float dAspect = dH/dW;
	const float aspect = (float)height/(float)width;

	x = (float)posX/(float)width;
	x *= dW;
	y = (float)posY/(float)height;
	y *= dH;

	if(aspect < dAspect)
	{
		offset = (float)width - (((float)height/dH) * dW);
		offset *= 0.5f;
	}
	else if(aspect > dAspect)
	{
		offset = (float)height - (((float)width/dW) * dH);
		offset *= 0.5f;
	}

	if(state)
		state->mouseUp(button, (int)x, (int)y);
}

//------------------------------------------------------------------------------
void Application::mouseMove(int posX, int posY)
{
	float x, y;
	float offset;
	const float dW = drawer->getDrawWidth();
	const float dH = drawer->getDrawHeight();
	const float dAspect = dH/dW;
	const float aspect = (float)height/(float)width;

	x = (float)posX/(float)width;
	x *= dW;
	y = (float)posY/(float)height;
	y *= dH;

	if(aspect < dAspect)
	{
		offset = (float)width - (((float)height/dH) * dW);
		offset *= 0.5f;
	}
	else if(aspect > dAspect)
	{
		offset = (float)height - (((float)width/dW) * dH);
		offset *= 0.5f;
	}

	if(state)
		state->mouseMove((int)x, (int)y);
}

//------------------------------------------------------------------------------
void Application::joyMove(int joystick, int axis, float value)
{
	if(state)
	{
		int axisVal = 0;

		if((axis == 0) && (value < 0.0f)) //Left
			axisVal = 3;
		else if((axis == 0) && (value > 0.0f)) //Right
			axisVal = 1;
		else if((axis == 0) && (value == 0.0f)) //Centre x-axis.
		{
			state->joyMove(keyManager.getAction(joystick, axisVal),
						   fabsf(value),
						   joystick,
						   3);
			axisVal = 1;
		}
		else if((axis == 1) && (value < 0.0f)) //Up
			axisVal = 0;
		else if((axis == 1) && (value >= 0.0f)) //Down
			axisVal = 2;
		else if((axis == 1) && (value == 0.0f)) //Centre y-axis.
		{
			state->joyMove(keyManager.getAction(joystick, axisVal),
						   fabsf(value),
						   joystick,
						   2);
			axisVal = 0;
		}
		else if((axis == 2) && (value < 0.0f)) //Trigger Left
			axisVal = 5;
		else if((axis == 2) && (value > 0.0f)) //Trigger Right
			axisVal = 4;
		else if((axis == 2) && (value == 0.0f)) //Centre Trigger.
		{
			state->joyMove(keyManager.getAction(joystick, axisVal),
						   fabsf(value),
						   joystick,
						   5);
			axisVal = 4;
		}
		else if((axis == 3) && (value < 0.0f)) //Left2
			axisVal = 9;
		else if((axis == 3) && (value >= 0.0f)) //Right2
			axisVal = 7;
		else if((axis == 3) && (value == 0.0f)) //Centre x-axis 2.
		{
			state->joyMove(keyManager.getAction(joystick, axisVal),
						   fabsf(value),
						   joystick,
						   9);
			axisVal = 7;
		}
		else if((axis == 4) && (value < 0.0f)) //Up2
			axisVal = 6;
		else if((axis == 4) && (value >= 0.0f)) //Down2
			axisVal = 8;
		else if((axis == 4) && (value == 0.0f)) //Centre y-axis 2.
		{
			state->joyMove(keyManager.getAction(joystick, axisVal),
						   fabsf(value),
						   joystick,
						   6);
			axisVal = 8;
		}
		state->joyMove(keyManager.getAction(joystick, axisVal),
					   fabsf(value),
					   joystick,
					   axisVal);
	}
}

//------------------------------------------------------------------------------
void Application::joyDown(int joystick, int button)
{
	if(state)
	{
		SDL_keysym nullKey;

		state->keyDown(keyManager.getAction(joystick, button+4), nullKey);
	}
}

//------------------------------------------------------------------------------
void Application::joyUp(int joystick, int button)
{
	if(state)
	{
		SDL_keysym nullKey;

		state->keyUp(keyManager.getAction(joystick, button+4), nullKey);
	}
}

//------------------------------------------------------------------------------
void Application::initKeyManager()
{
	//Player 1 movement.
	keyManager.addKeypress("P1Up", KeyManager::KeyUp);
	keyManager.addKeypress("P1Left", KeyManager::KeyLeft);
	keyManager.addKeypress("P1Right", KeyManager::KeyRight);
	keyManager.addKeypress("P1Down", KeyManager::KeyDown);

	//General.
	keyManager.addKeypress("Escape", KeyManager::KeyEscape);
	keyManager.addKeypress("Action", KeyManager::KeySpace);
	keyManager.addKeypress("Winter", KeyManager::KeyW);

	//Debug.
	keyManager.addKeypress("Debug", KeyManager::KeyK);
}

//------------------------------------------------------------------------------
void Application::loadGlyphs()
{
	ImageLoader imLoad;
	ImageData data;

	data = imLoad.loadImage(Fonts::fonta_png, Fonts::fonta_pngSize);
	drawer->loadGlyph(Drawer::a, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontb_png, Fonts::fontb_pngSize);
	drawer->loadGlyph(Drawer::b, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontc_png, Fonts::fontc_pngSize);
	drawer->loadGlyph(Drawer::c, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontd_png, Fonts::fontd_pngSize);
	drawer->loadGlyph(Drawer::d, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonte_png, Fonts::fonte_pngSize);
	drawer->loadGlyph(Drawer::e, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontf_png, Fonts::fontf_pngSize);
	drawer->loadGlyph(Drawer::f, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontg_png, Fonts::fontg_pngSize);
	drawer->loadGlyph(Drawer::g, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonth_png, Fonts::fonth_pngSize);
	drawer->loadGlyph(Drawer::h, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonti_png, Fonts::fonti_pngSize);
	drawer->loadGlyph(Drawer::i, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontj_png, Fonts::fontj_pngSize);
	drawer->loadGlyph(Drawer::j, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontk_png, Fonts::fontk_pngSize);
	drawer->loadGlyph(Drawer::k, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontl_png, Fonts::fontl_pngSize);
	drawer->loadGlyph(Drawer::l, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontm_png, Fonts::fontm_pngSize);
	drawer->loadGlyph(Drawer::m, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontn_png, Fonts::fontn_pngSize);
	drawer->loadGlyph(Drawer::n, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonto_png, Fonts::fonto_pngSize);
	drawer->loadGlyph(Drawer::o, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontp_png, Fonts::fontp_pngSize);
	drawer->loadGlyph(Drawer::p, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontq_png, Fonts::fontq_pngSize);
	drawer->loadGlyph(Drawer::q, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontr_png, Fonts::fontr_pngSize);
	drawer->loadGlyph(Drawer::r, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonts_png, Fonts::fonts_pngSize);
	drawer->loadGlyph(Drawer::s, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontt_png, Fonts::fontt_pngSize);
	drawer->loadGlyph(Drawer::t, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontu_png, Fonts::fontu_pngSize);
	drawer->loadGlyph(Drawer::u, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontv_png, Fonts::fontv_pngSize);
	drawer->loadGlyph(Drawer::v, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontw_png, Fonts::fontw_pngSize);
	drawer->loadGlyph(Drawer::w, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontx_png, Fonts::fontx_pngSize);
	drawer->loadGlyph(Drawer::x, data.data, data.width);

	data = imLoad.loadImage(Fonts::fonty_png, Fonts::fonty_pngSize);
	drawer->loadGlyph(Drawer::y, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontz_png, Fonts::fontz_pngSize);
	drawer->loadGlyph(Drawer::z, data.data, data.width);

	data = imLoad.loadImage(Fonts::font0_png, Fonts::font0_pngSize);
	drawer->loadGlyph(Drawer::Num0, data.data, data.width);

	data = imLoad.loadImage(Fonts::font1_png, Fonts::font1_pngSize);
	drawer->loadGlyph(Drawer::Num1, data.data, data.width);

	data = imLoad.loadImage(Fonts::font2_png, Fonts::font2_pngSize);
	drawer->loadGlyph(Drawer::Num2, data.data, data.width);

	data = imLoad.loadImage(Fonts::font3_png, Fonts::font3_pngSize);
	drawer->loadGlyph(Drawer::Num3, data.data, data.width);

	data = imLoad.loadImage(Fonts::font4_png, Fonts::font4_pngSize);
	drawer->loadGlyph(Drawer::Num4, data.data, data.width);

	data = imLoad.loadImage(Fonts::font5_png, Fonts::font5_pngSize);
	drawer->loadGlyph(Drawer::Num5, data.data, data.width);

	data = imLoad.loadImage(Fonts::font6_png, Fonts::font6_pngSize);
	drawer->loadGlyph(Drawer::Num6, data.data, data.width);

	data = imLoad.loadImage(Fonts::font7_png, Fonts::font7_pngSize);
	drawer->loadGlyph(Drawer::Num7, data.data, data.width);

	data = imLoad.loadImage(Fonts::font8_png, Fonts::font8_pngSize);
	drawer->loadGlyph(Drawer::Num8, data.data, data.width);

	data = imLoad.loadImage(Fonts::font9_png, Fonts::font9_pngSize);
	drawer->loadGlyph(Drawer::Num9, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontperiod_png, Fonts::fontperiod_pngSize);
	drawer->loadGlyph(Drawer::Period, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontcomma_png, Fonts::fontcomma_pngSize);
	drawer->loadGlyph(Drawer::Comma, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontplus_png, Fonts::fontplus_pngSize);
	drawer->loadGlyph(Drawer::Plus, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontminus_png, Fonts::fontminus_pngSize);
	drawer->loadGlyph(Drawer::Minus, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontexclaim_png, Fonts::fontexclaim_pngSize);
	drawer->loadGlyph(Drawer::Exclaim, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontquestion_png, Fonts::fontquestion_pngSize);
	drawer->loadGlyph(Drawer::Question, data.data, data.width);

	data = imLoad.loadImage(Fonts::fontcolon_png, Fonts::fontcolon_pngSize);
	drawer->loadGlyph(Drawer::Colon, data.data, data.width);
}
