//	Application.h - Class handling the various application bits and pieces.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef APPLICATION_H_
#define APPLICATION_H_

#include "NiallsDrawingLib/Drawer.h"
#include "PortAudioDevice.h"
#include "KeyManager.h"
#include "Snowflakes.h"
#include "ConfigLoader.h"
#include "Pickup.h"

#include "SDL.h"
#include <vector>
#include <string>
#include <map>

class GameState;
typedef std::vector<std::string> CmdLineParams;

//------------------------------------------------------------------------------
///	Class handling the various application bits and pieces.
class Application : public PortAudioCallback
{
  public:
	///	Constructor.
	/*!
		\param commandLine A vector containing all the command line parameters
		passed in.

		This will call SDL_Init() with the SDL_INIT_VIDEO and SDL_INIT_TIMER
		flags.  It will also open a window according to the command line
		arguments (e.g. fullscreen/windowed, geometry, bit depth etc.).
	 */
	Application(const CmdLineParams &commandLine);
	///	Destructor.
	/*!
		Will call SDL_Quit().
	 */
	~Application();

	///	Where the sound is generated and sent to the soundcard.
	void getAudio(float *input, float *output, int numSamples);

	///	The main event loop occurs inside this method.
	void eventLoop();

	///	Quits the game.
	void quit();

	///	Moves to the next game state.
	void nextState(const std::string newState, const std::string extra);

	///	Used to let us know when we're at the loud bit.
	void setLoudBit(bool val);
	///	Used to trigger stuff on the beat.
	void triggerBeat();

	///	Puts us in winter mode.
	void setWinter(bool val);
	///	Returns true if we're in winter mode.
	bool getWinter() const {return winter;};
	///	Returns true if winter mode has been unlocked.
	bool getWinterUnlocked() const {return winterUnlocked;};

	///	Sets which pickups to spawn at the start in the GamePlay state.
	void setStartPickups(bool *pickups);
	///	Returns which pickups to spawn at the start in the GamePlay state.
	bool *getStartPickups() {return startPickups;};
  private:
	///	Called when a key down event is received.
	void keyDown(const SDL_keysym& keyPressed);
	///	Called when a key up event is received.
	void keyUp(const SDL_keysym& keyPressed);

	///	Called when a mouse down event is received.
	void mouseDown(int button, int posX, int posY);
	///	Called when a mouse up event is received.
	void mouseUp(int button, int posX, int posY);
	///	Called when a mouse move event is received.
	void mouseMove(int posX, int posY);

	///	Called when a joystick axis changes.
	/*!
		\param joystick Which joystick's axis has changed.
		\param axis The joystick axis which has changed (only 0(x) and 1(y) are
		used.
		\param value The value -1->+1 of the axis which has changed.
	 */
	void joyMove(int joystick, int axis, float value);
	///	Called when a joystick button is pressed.
	void joyDown(int joystick, int button);
	///	Called when a joystick button is released.
	void joyUp(int joystick, int button);

	///	Initialises our key/joystick mappings.
	void initKeyManager();

	///	Helper method to create a jpeg file.
	void writeJpeg(unsigned char *pixels, const std::string& filename);

	///	Helper method to load drawer's glyphs from Fonts.cpp.
	void loadGlyphs();

	///	Our PortAudioDevice.
	PortAudioDevice portAudioDevice;

	///	We set this to true when it's time to quit the program.
	bool timeToQuit;
	///	We use this to determine if we need to repaint the screen immediately.
	Uint32 lastRepaint;

	///	The width of the window.
	int width;
	///	The height of the window.
	int height;
	///	True if we're not playing any sound.
	bool noSound;
	///	True if we're not playing the background music.
	bool noMusic;
	///	The path to files we may be rendering offline.
	std::string offlineFilename;
	///	The number of seconds we're going to render offline for.
	unsigned int offlineTime;

	///	Handles the mappings between keys/joysticks and actions.
	KeyManager keyManager;

	///	For playing with a joystick instead of the keyboard.
	std::vector<SDL_Joystick *> joysticks;
	///	Used to avoid 'stickiness' issues.
	std::vector<bool> joyUpIsDown;
	///	Used to avoid 'stickiness' issues.
	std::vector<bool> joyDownIsDown;
	///	Used to avoid 'stickiness' issues.
	std::vector<bool> joyLeftIsDown;
	///	Used to avoid 'stickiness' issues.
	std::vector<bool> joyRightIsDown;

	///	Draws everything on screen.
	Drawer *drawer;

	///	The current game state.
	GameState *state;

	///	Used to save when the player unlocks winter mode.
	ConfigLoader config;
	
	///	True if the player's returned to the herd.
	bool prevLoudBit;
	///	True if the player's returned to the herd.
	bool loudBit;
	///	Used to trigger stuff on the beat.
	bool beat;

	///	True if we're in winter mode.
	bool winter;
	///	True if winter mode has been unlocked.
	bool winterUnlocked;
	///	Snowflakes for winter mode.
	Snowflakes snowflakes;

	///	Which pickups to spawn at the start in the GamePlay state.
	bool startPickups[Pickup::NumTypes-1];
};

#endif
