//	EmperorSanter.cpp - An enormous santer that shoots lazors.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "EmperorSanter.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "GamePlay.h"
#include "Random.h"
#include "ThreeFloats.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
uint32_t EmperorSanter::image = 0;

//------------------------------------------------------------------------------
EmperorSanter::EmperorSanter():
life(8.0f),
coolDown(0.0f),
lazorCount(120),
insideNebula(false),
frozen(0.0f),
start(false)
{
	
}

//------------------------------------------------------------------------------
EmperorSanter::~EmperorSanter()
{
	
}

//------------------------------------------------------------------------------
void EmperorSanter::initialise(b2World& world, const TwoFloats& pos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2PolygonShape boundingBox;

	physicsWorld = &world;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.SetAsBox(pixels2Meters(17.0f*16.0f), pixels2Meters(30.0f*16.0f));
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.8f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);
}

//------------------------------------------------------------------------------
void EmperorSanter::takeDown()
{
	if(life <= 0.0f)
		physicsWorld->DestroyBody(physicsBody);
}

//------------------------------------------------------------------------------
void EmperorSanter::move(const TwoFloats& playerPos, GamePlay *gamePlay)
{
	float dist;
	b2Vec2 newForce;
	float santerAngle;
	float playerAngle;
	TwoFloats topLeft, bottomRight;
	b2Vec2 physicsPos = physicsBody->GetPosition();
	b2Vec2 physicsPlayer = twoFloats2B2Vec2(playerPos);
	TwoFloats pos(meters2Pixels(physicsPos.x), meters2Pixels(physicsPos.y));

	santerAngle = physicsBody->GetAngle();
	playerAngle = pos.getAngle(playerPos);

	topLeft.x = playerPos.x - 640.0f;
	topLeft.y = playerPos.y - 360.0f;
	bottomRight.x = playerPos.x + 640.0f;
	bottomRight.y = playerPos.y + 360.0f;

	if((!pos.isWithin(topLeft, bottomRight) || start) && (coolDown <= 0.0f))
	{
		dist = pos.getDistance(playerPos);

		if(!start && (dist > 384.0f))
		{
			newForce.x = sinf(playerAngle) * 20.0f;
			newForce.y = cosf(playerAngle) * 20.0f;

			physicsBody->ApplyLinearImpulse(newForce, physicsBody->GetWorldPoint(b2Vec2(1.0f, -0.5f)));

			start = false;
		}
	}

	if(coolDown > 0.0f)
		coolDown -= 0.01f;

	--lazorCount;
	if(lazorCount <= 0)
	{
		TwoFloats lazorPos;
		TwoFloats dir(sinf(santerAngle), -cosf(santerAngle));

		lazorPos = pos;
		lazorPos += (dir*(8.0f*16.0f));
		lazorPos += (dir.perpendicular()*6.0f*16.0f);
		gamePlay->addLazor(lazorPos, false);

		lazorPos -= (dir.perpendicular()*12.0f*16.0f);
		gamePlay->addLazor(lazorPos, false);

		lazorCount = 60 + (int)(Random::getGlobal().getRandomFloat() * 120);
	}

	if(frozen > 0.0f)
	{
		frozen -= 0.005f;

		if(!insideNebula)
		{
			float tempVal;

			tempVal = (1.0f-frozen) * 0.5f;
			tempVal += frozen * 2.0f;
			physicsBody->SetLinearDamping(tempVal);

			tempVal = (1.0f-frozen) * 0.05f;
			tempVal += frozen * 2.0f;
			physicsBody->SetAngularDamping(tempVal);
		}
	}
}

//------------------------------------------------------------------------------
void EmperorSanter::draw(Drawer *d, const TwoFloats& playerPos)
{
	ThreeFloats col;
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::emperorsanter_png, Images::emperorsanter_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - 512.0f;
	topLeft.y = playerPos.y - 360.0f - 512.0f;
	bottomRight.x = playerPos.x + 640.0f + 512.0f;
	bottomRight.y = playerPos.y + 360.0f + 512.0f;

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		const float freezeInterp = fakeLog(1.0f-frozen) * 0.5f;

		col.x = 0.5f + freezeInterp;
		col.y = 0.5f + freezeInterp;
		col.z = 1.0f;

		topLeft += 512.0f;
		d->drawImageRot(image,
						pos.x-topLeft.x-(32.0f*16.0f),
						pos.y-topLeft.y-(32.0f*16.0f),
						physicsBody->GetAngle() * (180.0f/3.14159265f),
						32.0f,
						32.0f,
						16.0f,
						16.0f,
						col.x,
						col.y,
						col.z);

		//Draw health bar.
		d->fillRect(pos.x-topLeft.x-256.0f,
					pos.y-topLeft.y-384.0f-16.0f,
					512.0f,
					48.0f,
					255,
					255,
					255,
					255);
		d->fillRect(pos.x-topLeft.x-256.0f+4,
					pos.y-topLeft.y-384.0f-16.0f+4,
					512.0f-8.0f,
					48.0f-8.0f,
					0,
					0,
					0,
					255);
		d->fillRect(pos.x-topLeft.x-256.0f+8,
					pos.y-topLeft.y-384.0f-16.0f+8,
					((life/8.0f) * 512.0f)-16.0f,
					48.0f-16.0f,
					255,
					0,
					0,
					255);
	}
}

//------------------------------------------------------------------------------
TwoFloats EmperorSanter::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void EmperorSanter::reduceHealth(float val)
{
	life -= val;
}

//------------------------------------------------------------------------------
void EmperorSanter::setCooldown()
{
	coolDown = 1.0f;
}

//------------------------------------------------------------------------------
void EmperorSanter::nebulaCollide(bool val)
{
	insideNebula = val;

	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void EmperorSanter::freeze()
{
	frozen = 1.0f;

	physicsBody->SetLinearDamping(2.0f);
	physicsBody->SetAngularDamping(2.0f);
}
