//	GamePlay.h - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAY_H_
#define GAMEPLAY_H_

#include "GameState.h"
#include "PlayerDonkey.h"
#include "Stars.h"
#include "c0xHeaders.h"
#include "GLES-Render.h"
#include "SanterExplosion.h"
#include "Asteroid.h"
#include "DonkeyHerd.h"
#include "Pickup.h"

#include "Box2D/Box2D.h"
#include <vector>

class Bomb;
class Lazor;
class Santer;
class Pickup;
class Nebula;
class Present;
class Elephant;
class Snowball;
class ShockWave;
class FlameSpark;
class LunarLander;
class WreckingBall;

///	Main gameplay state.
class GamePlay : public GameState,
				 public b2ContactListener,
				 public b2RayCastCallback
{
  public:
	///	Constructor.
	GamePlay(Application *app);
	///	Destructor.
	~GamePlay();

	///	Draws the state.
	void draw(Drawer *d);

	///	Called when a key down event is received.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);
	///	Called when a key up event is received.
	void keyUp(const std::string& action, const SDL_keysym& keyPressed);

	///	Called to handle collisions.
	void BeginContact(b2Contact *contact);
	///	Called to handle collisions.
	void EndContact(b2Contact *contact);
	///	Called to handle lazor collisions.
	float32 ReportFixture(b2Fixture *fixture,
						  const b2Vec2 &point,
						  const b2Vec2 &normal,
						  float32 fraction);

	///	Used to let us know when we're at the loud bit.
	void setLoudBit(bool val);
	///	Used to trigger stuff on the beat.
	void triggerBeat();

	///	Used to add a lazor to the scene.
	void addLazor(const TwoFloats& start, bool alsoTargetSanters);
	///	Used to add a present projectile to the scene.
	void addPresent(const TwoFloats& start);
	///	Used to add a flame spark to the scene.
	void addSpark(const TwoFloats& pos, const TwoFloats& dir);

	///	Returns the name of this state.
	const std::string getName() const {return "GamePlay";};
  private:
	///	Helper method to handle bomb explosions.
	void bombExplosion(int index);
	///	Helper method to handle pickup collection.
	void collectPickup(Pickup *pickup);
	///	Helper method to handle activatable pickups.
	void activatePickup(Pickup::PickupType pickup);
	///	Helper method to test lazor collisions.
	void lazorCollisions();
	///	Helper method. Returns a random position outside the screen.
	TwoFloats getOutsideScreenPos();

	///	The heart image.
	static uint32_t heartImage;
	///	The game win text.
	static uint32_t ohYeah;
	///	The game win text.
	static uint32_t backInTheHerd;
	///	The compass arrow image.
	static uint32_t arrowImage;
	///	The pickup UI image.
	static uint32_t pickupUI;
	///	The font we use.
	static uint32_t font;

	///	The player's donkey.
	PlayerDonkey player;
	///	The santers in play.
	std::vector<StdSharedPtr<Santer>::type> santers;
	///	The donkey herd.
	DonkeyHerd donkeyHerd;
	///	Any pickups in play.
	std::vector<StdSharedPtr<Pickup>::type> pickups;
	///	Any elephants in play.
	std::vector<StdSharedPtr<Elephant>::type> elephants;
	///	Any wrecking balls in play.
	std::vector<StdSharedPtr<WreckingBall>::type> wreckingBalls;
	///	Any bombs in play.
	std::vector<StdSharedPtr<Bomb>::type> bombs;
	///	Any lazors in play.
	std::vector<StdSharedPtr<Lazor>::type> lazors;
	///	Any presents in play.
	std::vector<StdSharedPtr<Present>::type> presents;
	///	Any snowballs in play.
	std::vector<StdSharedPtr<Snowball>::type> snowballs;
	///	Any flame sparks in play.
	std::vector<StdSharedPtr<FlameSpark>::type> flameSparks;

	///	Any asteroids in play.
	std::vector<StdSharedPtr<Asteroid>::type> asteroids;
	///	Any Santer explosions currently active.
	std::vector<SanterExplosion> explosions;
	///	Any shock waves currently active.
	std::vector<ShockWave> shockWaves;
	///	Any nebulae in play.
	std::vector<StdSharedPtr<Nebula>::type> nebulae;
	///	Any lunar landers in play.
	std::vector<StdSharedPtr<LunarLander>::type> lunarLanders;

	///	The background stars.
	Stars stars;

	///	The Box2D world instance.
	b2World physicsWorld;
	///	Some constants used by Box2D.
	const float timeStep;
	///	Some constants used by Box2D.
	const float velocityIt;
	///	Some constants used by Box2D.
	const float positionIt;

	///	Used to determine when to add new santers.
	int santerCount;
	///	Used to determine when to add new santers.
	int santerToggle;
	///	Used to determine when to add new santers.
	int maxSanters;
	///	There should only be one emperor santer per playthrough.
	bool emperor;

	///	Used to ensure certain pickups are only generated once.
	int pickupsMax[Pickup::NumTypes];
	///	Used to determine when to add new pickups.
	int pickupCount;
	///	Used to determine when to add new pickups.
	int remainingPickups;
	///	Used to ensure the player gets at least one activatable pickup.
	int numActivatable;
	///	Which activatable pickup we have currently.
	Pickup::PickupType activatablePickup;
	///	The activatable pickup we're about to equip.
	Pickup::PickupType newActivatable;
	///	The cooldown time for the current activatable pickup.
	float activateCooldown;

	///	Any text to display at the bottom of the screen.
	std::string instructionText;
	///	How long to keep the current instructionText on screen.
	int instructionCount;
	///	If we've displayed the starting instruction yet.
	static bool startInstruction;
	///	If we've displayed the initial santer instruction yet.
	static bool santerInstruction;
	///	If we've displayed the nebula instruction yet.
	static bool nebulaInstruction;
	///	If we've displayed the king santer instruction yet.
	static bool kingInstruction;
	///	If we've displayed the santer slay instruction yet.
	static bool slayInstruction;
	///	If we've displayed the ice santer instruction yet.
	static bool iceInstruction;
	///	If we've displayed the winter instruction yet.
	static bool winterInstruction;

	///	True if the player's returned to the herd.
	bool gameWin;
	///	Used to signal we need to connect the player to this donkey.
	DonkeyHerd::Donkey *createJoint;

	///	True if we're currently in the loud bit.
	bool loudBit;
	///	Used to emphasize the beats.
	float beat;
	///	Used to add an apple when we transition in/out of the loud bit.
	bool addApple;

	///	Used to draw the arrow pointing towards the herd.
	bool compass;
	///	Used to add an Elephant to the world.
	bool addElephant;
	///	Used to add a WreckingBall to the world.
	bool addWreckingBall;

	///	Used to freeze all non-player bodies.
	float stopTime;

	///	Used to display information about the pickups the player's picked up.
	float pickupDisplay;
	///	Used to display information about the pickups the player's picked up.
	std::string pickupName;
	///	Used to display information about the pickups the player's picked up.
	std::string pickupDesc;

	///	Used to automatically spawn a compass if the player's lost.
	float lostCount;
	///	Used to automatically spawn a compass if the player's lost.
	bool spawnedLostCompass;

	///	Used by the lazor raycasting.
	int lazorIndex;

	///	Used to make sure the player doesn't accidentally quit to title too early.
	int titleCount;

	///	For debugging.
	GLESDebugDraw debugDrawer;
	///	For debugging.
	bool debugMode;
};

#endif
