//	KingSanter.cpp - A big santer with a health bar.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "KingSanter.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "ThreeFloats.h"
#include "MiscHelper.h"

//------------------------------------------------------------------------------
uint32_t KingSanter::imageBody = 0;

//------------------------------------------------------------------------------
KingSanter::KingSanter():
armRot(0.0f),
life(4.0f),
coolDown(0.0f),
chargeCount(0),
insideNebula(false),
frozen(0.0f),
start(false)
{
	
}

//------------------------------------------------------------------------------
KingSanter::~KingSanter()
{
	
}

//------------------------------------------------------------------------------
void KingSanter::initialise(b2World& world, const TwoFloats& pos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2PolygonShape boundingBox;

	physicsWorld = &world;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.SetAsBox(pixels2Meters(96.0f*2.0f), pixels2Meters(40.0f*2.0f));
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.8f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);

	physicsBody->SetTransform(bodyDef.position, 1.570796f);
}

//------------------------------------------------------------------------------
void KingSanter::takeDown()
{
	if(life <= 0.0f)
		physicsWorld->DestroyBody(physicsBody);
}

//------------------------------------------------------------------------------
void KingSanter::move(const TwoFloats& playerPos, GamePlay *gamePlay)
{
	float dist;
	b2Vec2 newForce;
	float KingSanterAngle;
	float playerAngle;
	TwoFloats topLeft, bottomRight;
	b2Vec2 physicsPos = physicsBody->GetPosition();
	b2Vec2 physicsPlayer = twoFloats2B2Vec2(playerPos);
	TwoFloats pos(meters2Pixels(physicsPos.x), meters2Pixels(physicsPos.y));

	KingSanterAngle = physicsBody->GetAngle();
	playerAngle = pos.getAngle(playerPos);

	topLeft.x = playerPos.x - 640.0f;
	topLeft.y = playerPos.y - 360.0f;
	bottomRight.x = playerPos.x + 640.0f;
	bottomRight.y = playerPos.y + 360.0f;

	if((!pos.isWithin(topLeft, bottomRight) || start) && (coolDown <= 0.0f))
	{
		dist = pos.getDistance(playerPos);

		if(!start && (dist > 384.0f))
		{
			if(chargeCount > 0)
			{
				newForce.x = sinf(playerAngle) * 10.0f;
				newForce.y = cosf(playerAngle) * 10.0f;
			}
			else
			{
				newForce.x = sinf(playerAngle) * 60.0f;
				newForce.y = cosf(playerAngle) * 60.0f;
				chargeCount = 120;
			}
			physicsBody->ApplyLinearImpulse(newForce, physicsBody->GetWorldPoint(b2Vec2(1.0f, -0.5f)));
			--chargeCount;

			start = false;
		}
	}

	if(coolDown > 0.0f)
		coolDown -= 0.01f;

	if(frozen > 0.0f)
	{
		frozen -= 0.005f;

		if(!insideNebula)
		{
			float tempVal;

			tempVal = (1.0f-frozen) * 0.5f;
			tempVal += frozen * 2.0f;
			physicsBody->SetLinearDamping(tempVal);

			tempVal = (1.0f-frozen) * 0.05f;
			tempVal += frozen * 2.0f;
			physicsBody->SetAngularDamping(tempVal);
		}
	}
}

//------------------------------------------------------------------------------
void KingSanter::draw(Drawer *d, const TwoFloats& playerPos)
{
	ThreeFloats col;
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	if(!KingSanter::imageBody)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::kingsanter_png, Images::kingsanter_pngSize);
		KingSanter::imageBody = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::santerarm_png, Images::santerarm_pngSize);
		Santer::imageArm = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - 256.0f;
	topLeft.y = playerPos.y - 360.0f - 256.0f;
	bottomRight.x = playerPos.x + 640.0f + 256.0f;
	bottomRight.y = playerPos.y + 360.0f + 256.0f;

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		const float freezeInterp = fakeLog(1.0f-frozen) * 0.5f;

		col.x = 0.5f + freezeInterp;
		col.y = 0.5f + freezeInterp;
		col.z = 1.0f;

		topLeft += 256.0f;
		d->drawImageRot(KingSanter::imageBody,
						pos.x-topLeft.x-256.0f,
						pos.y-topLeft.y-256.0f,
						physicsBody->GetAngle() * (180.0f/3.14159265f),
						32.0f,
						32.0f,
						8.0f,
						8.0f,
						col.x,
						col.y,
						col.z);

		d->drawImageRot(Santer::imageArm,
						pos.x-topLeft.x-256.0f,
						pos.y-topLeft.y-256.0f,
						armRot * (180.0f/3.14159265f),
						30.0f,
						34.0f,
						8.0f,
						8.0f,
						col.x,
						col.y,
						col.z);

		//Draw health bar.
		d->fillRect(pos.x-topLeft.x-128.0f,
					pos.y-topLeft.y-128.0f-16.0f,
					256.0f,
					24.0f,
					255,
					255,
					255,
					255);
		d->fillRect(pos.x-topLeft.x-128.0f+4,
					pos.y-topLeft.y-128.0f-16.0f+4,
					256.0f-8.0f,
					24.0f-8.0f,
					0,
					0,
					0,
					255);
		d->fillRect(pos.x-topLeft.x-128.0f+8,
					pos.y-topLeft.y-128.0f-16.0f+8,
					((life/4.0f) * 256.0f)-16.0f,
					24.0f-16.0f,
					255,
					0,
					0,
					255);
	}

	armRot += 0.25f;
	if(armRot > 6.283185f)
		armRot = fmodf(armRot, 6.283185f);
}

//------------------------------------------------------------------------------
TwoFloats KingSanter::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void KingSanter::reduceHealth(float val)
{
	life -= val;
}

//------------------------------------------------------------------------------
void KingSanter::setCooldown()
{
	coolDown = 1.0f;
}

//------------------------------------------------------------------------------
void KingSanter::nebulaCollide(bool val)
{
	insideNebula = val;

	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void KingSanter::freeze()
{
	frozen = 1.0f;

	physicsBody->SetLinearDamping(2.0f);
	physicsBody->SetAngularDamping(2.0f);
}
