//	LunarLander.cpp - A lunar lander that will attack if provoked.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "LunarLander.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "GamePlay.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t LunarLander::image = 0;
uint32_t LunarLander::text[4] = {0, 0, 0, 0};

//------------------------------------------------------------------------------
LunarLander::LunarLander():
threeStrikes(0),
laserCount(120),
textCount(0)
{
	orbitIndex = Random::getGlobal().getRandomFloat();
	if(Random::getGlobal().getRandomFloat() > 0.5f)
		orbitSpeed = 1.0f + Random::getGlobal().getRandomFloat();
	else
		orbitSpeed = -1.0f - Random::getGlobal().getRandomFloat();
}

//------------------------------------------------------------------------------
LunarLander::~LunarLander()
{
	
}

//------------------------------------------------------------------------------
void LunarLander::initialise(b2World& world, const TwoFloats& pos)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2MouseJointDef jointDef;
	b2CircleShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters((32.0f-10.0f) * 4.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);

	physicsBody->SetTransform(bodyDef.position, 0.0f);
	physicsBody->SetLinearDamping(0.5f);
}

//------------------------------------------------------------------------------
void LunarLander::move(GamePlay *gamePlay)
{
	TwoFloats dir;
	//TwoFloats centre;
	//TwoFloats pos = getPosition();

	dir.x = sinf(orbitIndex * 6.283185f) * 10.0f;
	dir.y = -cosf(orbitIndex * 6.283185f) * 10.0f;

	//centre = pos - dir;
	dir = dir.perpendicular();
	physicsBody->ApplyLinearImpulse(b2Vec2(pixels2Meters(dir.x),
										   pixels2Meters(dir.y)),
									physicsBody->GetWorldCenter());

	orbitIndex += orbitSpeed * 0.00005f;
	if(orbitIndex > 1.0f)
		orbitIndex -= 1.0f;
	else if(orbitIndex < 0.0f)
		orbitIndex += 1.0f;

	if(threeStrikes >= 3)
	{
		--laserCount;
		if(laserCount <= 0)
		{
			gamePlay->addLazor(getPosition(), true);

			laserCount = 60 + (int)(Random::getGlobal().getRandomFloat() * 120);
		}
	}
	
}

//------------------------------------------------------------------------------
void LunarLander::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(b2Vec22TwoFloats(physicsBody->GetPosition()));

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::lunarlander_png, Images::lunarlander_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::landertext0_png, Images::landertext0_pngSize);
		text[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::landertext1_png, Images::landertext1_pngSize);
		text[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::landertext2_png, Images::landertext2_pngSize);
		text[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::landertext3_png, Images::landertext3_pngSize);
		text[3] = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f - (32.0f * 4.0f);
	topLeft.y = playerPos.y - 360.0f - (32.0f * 4.0f);
	bottomRight.x = playerPos.x + 640.0f + (32.0f * 4.0f);
	bottomRight.y = playerPos.y + 360.0f + (32.0f * 4.0f);

	pos.x = meters2Pixels(pos.x);
	pos.y = meters2Pixels(pos.y);
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		topLeft += (32.0f * 4.0f);
		d->drawImage(image,
					 pos.x - topLeft.x - (32.0f * 4.0f),
					 pos.y - topLeft.y - (32.0f * 4.0f),
					 4.0f,
					 4.0f);
	}
	else
		topLeft += (32.0f * 4.0f);

	if(textCount > 0)
	{
		d->drawImage(text[threeStrikes],
					 pos.x - topLeft.x - (32.0f * 4.0f) + (23.0f * 4.0f),
					 pos.y - topLeft.y - (32.0f * 4.0f) - (10.0f * 4.0f),
					 4.0f,
					 4.0f);

		--textCount;
		if(textCount <= 0)
			++threeStrikes;
	}
}

//------------------------------------------------------------------------------
TwoFloats LunarLander::getPosition() const
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void LunarLander::nebulaCollide(bool val)
{
	if(val)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void LunarLander::actorCollide()
{
	if(!textCount && (threeStrikes < 4))
		textCount = 300;
}
