//	NewtonianActor.h - Base class for all actors using newtonian physics.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef NEWTONIANACTOR_H_
#define NEWTONIANACTOR_H_

#include "TwoFloats.h"

///	Base class for all actors using newtonian physics.
class NewtonianActor
{
  public:
	///	Constructor.
	NewtonianActor();
	///	Destructor.
	virtual ~NewtonianActor();

	///	Sets the actor's position, disregarding physics.
	void setPosition(const TwoFloats& pos);
	///	Returns the actor's position.
	const TwoFloats& getPosition() const {return position;};

	///	Sets the actor's direction, disregarding physics.
	void setDirection(const TwoFloats& dir);
	///	Returns the actor's direction.
	const TwoFloats& getDirection() const {return direction;};

	///	Returns the actor's angle of rotation.
	const float getRotation() const {return rotation;};

	///	Returns the actor's speed.
	const TwoFloats& getSpeed() const {return speed;};
	///	Sets the actor's maximum possible speed.
	void setMaxSpeed(float val);
	///	Returns the actor's maximum possible speed.
	const float getMaxSpeed() const {return maxSpeed;};

	///	Applies a force to the actor.
	void applyForce(const TwoFloats& force);

	///	Implemented by subclasses to handle the actor's motion.
	/*!
		Subclasses should call update() from within this method.
	 */
	virtual void move() = 0;
  protected:
	///	Updates the actor's position and rotation according to its momentum.
	void update();
  private:
	///	The actor's position.
	TwoFloats position;
	///	The actor's direction vector.
	TwoFloats direction;
	///	The actor's angle of rotation.
	float rotation;
	///	The actor's maximum possible speed.
	float maxSpeed;

	///	The actor's speed vector.
	TwoFloats speed;
	///	The rate at which speed falls off.
	float fallOff;
};

#endif
