//	Pickup.cpp - Class representing a pickup.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Pickup.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"

//------------------------------------------------------------------------------
uint32_t Pickup::images[NumTypes] = {0,
									 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
									 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint32_t Pickup::corona = 0;
uint32_t Pickup::radarBack = 0;
uint32_t Pickup::radarArrow = 0;

//------------------------------------------------------------------------------
Pickup::Pickup(int type, const TwoFloats& pos, b2World& world, bool waitVal):
pickupType((PickupType)type),
colourInterp(0.0f),
dead(false),
wait(waitVal ? 1.0f : 0.0f)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2CircleShape boundingBox;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.m_p.Set(0.0f, 0.0f);
	boundingBox.m_radius = pixels2Meters(32.0f);
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	//physicsBody->SetTransform(bodyDef.position, 0.0f);
	physicsBody->SetLinearDamping(0.5f);

	//Set up corona colours.
	colours[0] = ThreeFloats(1.0f, 0.0f, 0.0f);
	colours[1] = ThreeFloats(1.0f, 0.5f, 0.0f);
	colours[2] = ThreeFloats(1.0f, 1.0f, 0.0f);
	colours[3] = ThreeFloats(0.0f, 0.5f, 0.0f);
	colours[4] = ThreeFloats(0.0f, 0.0f, 1.0f);
	colours[5] = ThreeFloats(0.5f, 0.0f, 1.0f);
	colours[6] = ThreeFloats(1.0f, 0.0f, 1.0f);
}

//------------------------------------------------------------------------------
Pickup::~Pickup()
{

}

//------------------------------------------------------------------------------
void Pickup::move(const TwoFloats& playerPos)
{
	float amount;
	b2Vec2 newForce;
	float playerAngle;
	TwoFloats pos = getPosition();
	float dist = pos.getDistance(playerPos);

	playerAngle = pos.getAngle(playerPos);

	if(wait <= 0.0f)
	{
		if(fabsf(dist) > 0.0f)
			amount = (1.0f/dist) * 200.0f;
		else
			amount = 1.0f;
		if(amount < 0.05f)
			amount = 0.05f;
		newForce.x = sinf(playerAngle) * amount;
		newForce.y = cosf(playerAngle) * amount;
		physicsBody->ApplyLinearImpulse(newForce, physicsBody->GetWorldPoint(b2Vec2(1.0f, -0.5f)));
	}

	if(wait > 0.0f)
		wait -= 0.002f;
}

//------------------------------------------------------------------------------
void Pickup::draw(Drawer *d, const TwoFloats& playerPos)
{
	TwoFloats topLeft, bottomRight;
	TwoFloats pos(getPosition());
	ThreeFloats col;
	int prev, next;
	float tempf;

	if(!images[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::pickupapple_png, Images::pickupapple_pngSize);
		images[Apple] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupchilli_png, Images::pickupchilli_pngSize);
		images[Chilli] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupelephant_png, Images::pickupelephant_pngSize);
		images[Elephant] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupspeed_png, Images::pickupspeed_pngSize);
		images[SpeedUp] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupcompass_png, Images::pickupcompass_pngSize);
		images[Compass] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupmagnifyingglass_png, Images::pickupmagnifyingglass_pngSize);
		images[MagnifyingGlass] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupfire_png, Images::pickupfire_pngSize);
		images[Fire] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupwreckingball_png, Images::pickupwreckingball_pngSize);
		images[WreckingBall] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuphomingbeacon_png, Images::pickuphomingbeacon_pngSize);
		images[HomingBeacon] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupicecube_png, Images::pickupicecube_pngSize);
		images[IceCube] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupscarf_png, Images::pickupscarf_pngSize);
		images[Scarf] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupbuffer_png, Images::pickupbuffer_pngSize);
		images[Buffer] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupbomb_png, Images::pickupbomb_pngSize);
		images[Bomb] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupturbo_png, Images::pickupturbo_pngSize);
		images[Turbo] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupcake_png, Images::pickupcake_pngSize);
		images[Cake] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuprock_png, Images::pickuprock_pngSize);
		images[Rock] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuppigeon_png, Images::pickuppigeon_pngSize);
		images[Pigeon] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupshield_png, Images::pickupshield_pngSize);
		images[Shield] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupunicornhorn_png, Images::pickupunicornhorn_pngSize);
		images[UnicornHorn] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupsnowball_png, Images::pickupsnowball_pngSize);
		images[Snowball] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupstopwatch_png, Images::pickupstopwatch_pngSize);
		images[Stopwatch] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::corona_png, Images::corona_pngSize);
		corona = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupradarback_png, Images::pickupradarback_pngSize);
		radarBack = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupradararrow_png, Images::pickupradararrow_pngSize);
		radarArrow = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	topLeft.x = playerPos.x - 640.0f;
	topLeft.y = playerPos.y - 360.0f;
	bottomRight.x = playerPos.x + 640.0f;
	bottomRight.y = playerPos.y + 360.0f;
	
	if(pos.isWithin(topLeft, bottomRight))
	{
		prev = (int)colourInterp;
		next = (prev+1)%7;
		tempf = colourInterp - (float)prev;
		col = (1.0f-tempf) * colours[prev];
		col += tempf * colours[next];

		d->setBlendMode(Drawer::Additive);
		d->drawImage(corona,
					 pos.x - topLeft.x - (8.0f*8.0f),
					 pos.y - topLeft.y - (8.0f*8.0f),
					 8.0f,
					 8.0f,
					 col.x,
					 col.y,
					 col.z);
		d->setBlendMode(Drawer::AlphaBlend);

		d->drawImage(images[pickupType],
					 pos.x - topLeft.x - (8.0f*4.0f),
					 pos.y - topLeft.y - (8.0f*4.0f),
					 4.0f,
					 4.0f);
	}
	else //Draw the 'radar' arrow thingy.
	{
		float arrowAngle;
		TwoFloats radarPos;
		TwoFloats arrowPos;
		const float edgeDist = 16.0f * 4.0f;

		radarPos = pos - playerPos;
		arrowAngle = atan2f(radarPos.y, radarPos.x);
		arrowPos = radarPos.normal() * (13.0f*4.0f);
		radarPos.x += 640.0f;
		radarPos.y += 360.0f;

		if(radarPos.x < 0.0f)
			radarPos.x = edgeDist;
		else if(radarPos.x > 1280.0f)
			radarPos.x = (1280.0f-edgeDist);

		if(radarPos.y < 0.0f)
			radarPos.y = edgeDist;
		else if(radarPos.y > 720.0f)
			radarPos.y = (720.0f-edgeDist);

		//Draw back.
		d->drawImage(radarBack,
					 radarPos.x - edgeDist,
					 radarPos.y - edgeDist,
					 4.0f,
					 4.0f);

		//Draw arrow.
		d->drawImageRot(radarArrow,
						radarPos.x + arrowPos.x - (8.0f*4.0f),
						radarPos.y + arrowPos.y - (8.0f*4.0f),
						arrowAngle * (180.0f/3.14159265f),
						8.0f,
						8.0f,
						4.0f,
						4.0f);

		//Draw pickup.
		d->drawImage(images[pickupType],
					 radarPos.x - (8.0f*4.0f),
					 radarPos.y - (8.0f*4.0f),
					 4.0f,
					 4.0f);
	}

	colourInterp += 0.33f;
	if(colourInterp >= 7.0f)
		colourInterp -= 7.0f;
}

//------------------------------------------------------------------------------
void Pickup::setDead()
{
	dead = true;
}

//------------------------------------------------------------------------------
TwoFloats Pickup::getPosition()
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
const std::string Pickup::getActualName() const
{
	std::string retval;

	switch(pickupType)
	{
		case Apple:
			retval = "Apple";
			break;
		case Chilli:
			retval = "Chilli";
			break;
		case Elephant:
			retval = "Elephant";
			break;
		case SpeedUp:
			retval = "Speedy Boots";
			break;
		case Compass:
			retval = "Compass";
			break;
		case MagnifyingGlass:
			retval = "Magnifying Glass";
			break;
		case Fire:
			retval = "Fire";
			break;
		case WreckingBall:
			retval = "Wrecking Ball";
			break;
		case HomingBeacon:
			retval = "Homing Beacon";
			break;
		case IceCube:
			retval = "Ice Cube";
			break;
		case Scarf:
			retval = "Wooly Scarf";
			break;
		case Buffer:
			retval = "Buffer";
			break;
		case Bomb:
			retval = "Bomb";
			break;
		case Turbo:
			retval = "Turbo";
			break;
		case Cake:
			retval = "Yummy Cake";
			break;
		case Rock:
			retval = "Rock";
			break;
		case Pigeon:
			retval = "Homing Pigeon";
			break;
		case Shield:
			retval = "Shield";
			break;
		case UnicornHorn:
			retval = "Unicorn Horn";
			break;
		case Snowball:
			retval = "Snowball";
			break;
		case Stopwatch:
			retval = "Stopwatch";
			break;
	}

	return retval;
}

//------------------------------------------------------------------------------
const std::string Pickup::getDescription() const
{
	std::string retval;

	switch(pickupType)
	{
		case Apple:
			retval = "Heals donkey";
			break;
		case Chilli:
			retval = "Flamy fire breath";
			break;
		case Elephant:
			retval = "Yr Friendly Elephant Pal";
			break;
		case SpeedUp:
			retval = "Go faster stripes";
			break;
		case Compass:
			retval = "Points towards the herd";
			break;
		case MagnifyingGlass:
			retval = "Tiny donkey";
			break;
		case Fire:
			retval = "Infinite bottom fire";
			break;
		case WreckingBall:
			retval = "A magic wrecking ball";
			break;
		case HomingBeacon:
			retval = "So the herd can find you";
			break;
		case IceCube:
			retval = "Freeze santers on touch";
			break;
		case Scarf:
			retval = "Reduces damage taken by donkey";
			break;
		case Buffer:
			retval = "Pushes away all nearby objects";
			break;
		case Bomb:
			retval = "Drops a bomb";
			break;
		case Turbo:
			retval = "A quick blast of speed";
			break;
		case Cake:
			retval = "Heals donkey";
			break;
		case Rock:
			retval = "Generates an asteroid field";
			break;
		case Pigeon:
			retval = "Jumps donkey halfway to herd";
			break;
		case Shield:
			retval = "A temporary barrier";
			break;
		case UnicornHorn:
			retval = "Invincibility";
			break;
		case Snowball:
			retval = "Fire snowballs at the santers";
			break;
		case Stopwatch:
			retval = "Freeze time";
			break;
	}

	return retval;
}

//------------------------------------------------------------------------------
void Pickup::loadImages(Drawer *d)
{
	if(!images[0])
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::pickupapple_png, Images::pickupapple_pngSize);
		images[Apple] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupchilli_png, Images::pickupchilli_pngSize);
		images[Chilli] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupelephant_png, Images::pickupelephant_pngSize);
		images[Elephant] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupspeed_png, Images::pickupspeed_pngSize);
		images[SpeedUp] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupcompass_png, Images::pickupcompass_pngSize);
		images[Compass] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupmagnifyingglass_png, Images::pickupmagnifyingglass_pngSize);
		images[MagnifyingGlass] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupfire_png, Images::pickupfire_pngSize);
		images[Fire] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupwreckingball_png, Images::pickupwreckingball_pngSize);
		images[WreckingBall] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuphomingbeacon_png, Images::pickuphomingbeacon_pngSize);
		images[HomingBeacon] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupicecube_png, Images::pickupicecube_pngSize);
		images[IceCube] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupscarf_png, Images::pickupscarf_pngSize);
		images[Scarf] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupbuffer_png, Images::pickupbuffer_pngSize);
		images[Buffer] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupbomb_png, Images::pickupbomb_pngSize);
		images[Bomb] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupturbo_png, Images::pickupturbo_pngSize);
		images[Turbo] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupcake_png, Images::pickupcake_pngSize);
		images[Cake] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuprock_png, Images::pickuprock_pngSize);
		images[Rock] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickuppigeon_png, Images::pickuppigeon_pngSize);
		images[Pigeon] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupshield_png, Images::pickupshield_pngSize);
		images[Shield] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupunicornhorn_png, Images::pickupunicornhorn_pngSize);
		images[UnicornHorn] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupsnowball_png, Images::pickupsnowball_pngSize);
		images[Snowball] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupstopwatch_png, Images::pickupstopwatch_pngSize);
		images[Stopwatch] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::corona_png, Images::corona_pngSize);
		corona = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupradarback_png, Images::pickupradarback_pngSize);
		radarBack = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::pickupradararrow_png, Images::pickupradararrow_pngSize);
		radarArrow = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}
}
