//	Pickup.h - Class representing a pickup.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PICKUP_H_
#define PICKUP_H_

#include "TwoFloats.h"
#include "Collidable.h"
#include "ThreeFloats.h"

#include "Box2D/Box2D.h"
#include <stdint.h>

class Drawer;

///	Class representing a pickup.
class Pickup : public Collidable
{
  public:
	///	Constructor.
	/*!
		\param type Which type of pickup to spawn.
		\param pos The position to spawn it at.
		\param world The Box2D world to spawn it in.
		\param waitVal Used to ensure dropped activatable pickups don't get
		immediately picked back up by the player.
	 */
	Pickup(int type, const TwoFloats& pos, b2World& world, bool waitVal = false);
	///	Destructor.
	~Pickup();

	///	Moves the pickup.
	void move(const TwoFloats& playerPos);
	///	Draws the pickup.
	void draw(Drawer *d, const TwoFloats& playerPos);

	///	Informs the main thread that the pickup should be deleted.
	void setDead();
	///	Whether or not this pickup should be deleted.
	bool isDead() const {return dead;};

	///	Returns true if the player can pick up this pickup.
	bool getWait() const {return (wait <= 0.0f);};

	///	Returns the pickup's position.
	TwoFloats getPosition();
	///	Returns the pickup's physics body.
	b2Body *getBody() const {return physicsBody;};
	///	Implemented for collision purposes.
	const std::string getName() const {return "Pickup";};

	///	Returns the actual name of this particular pickup.
	const std::string getActualName() const;
	///	Returns the short description of this particular pickup.
	const std::string getDescription() const;

	///	Enum representing all the pickup types.
	enum PickupType
	{
		Apple = 0,
		Chilli,
		Elephant,
		SpeedUp,
		Compass,
		MagnifyingGlass,
		Fire,
		WreckingBall,
		HomingBeacon,
		IceCube,
		Scarf,
		Buffer,
		Bomb,
		Turbo,
		Cake,
		Rock,
		Pigeon,
		Shield,
		UnicornHorn,
		Snowball,
		Stopwatch,

		NumTypes
	};

	///	Returns the pickup's type.
	PickupType getType() const {return pickupType;};

	///	Used to load all the pickup images.
	static void loadImages(Drawer *d);
	///	The various pickup images.
	static uint32_t images[NumTypes];
  private:
	///	The corona image drawn behind the pickup.
	static uint32_t corona;
	///	The 'pickup radar' images.
	static uint32_t radarBack;
	///	The 'pickup radar' images.
	static uint32_t radarArrow;

	///	The pickup's image.
	PickupType pickupType;

	///	The pickup's Box2D body.
	b2Body *physicsBody;

	///	Used to colour cycle the corona.
	float colourInterp;
	///	Used to colour cycle the corona.
	ThreeFloats colours[7];

	///	Whether or not this pickup is ready to be deleted.
	bool dead;
	/// Used to ensure the player doesn't instantly pick up a pickup they've just dropped.
	float wait;
};

#endif
