//	PlayerDonkey.cpp - The player donkey.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PlayerDonkey.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "NiallsDrawingLib/OpenGLDrawer.h"
#include "Images.h"
#include "Box2DHelpers.h"
#include "Random.h"
#include "MiscHelper.h"
#include "ThreeFloats.h"
#include "GamePlay.h"

//------------------------------------------------------------------------------
uint32_t PlayerDonkey::image = 0;
uint32_t PlayerDonkey::flameImages[4] = {0, 0, 0, 0};
uint32_t PlayerDonkey::chilliImages[4] = {0, 0, 0, 0};
uint32_t PlayerDonkey::bootsImage = 0;
uint32_t PlayerDonkey::beaconImage = 0;
uint32_t PlayerDonkey::waveImage = 0;
uint32_t PlayerDonkey::hornImage = 0;
uint32_t PlayerDonkey::shieldImage = 0;
uint32_t PlayerDonkey::scarfImage = 0;

//------------------------------------------------------------------------------
PlayerDonkey::PlayerDonkey():
starsCount(0),
life(1.0f),
chilliFlames(true),
shield(0),
speed(1.0f),
flameIndex(0),
flameCount(5),
sparkCount(0),
chilli(false),
small(false),
updateScale(false),
fire(false),
homingBeacon(false),
beaconIndex(0.0f),
teleportIndex(1.0f),
invincible(0.0f),
shieldIndex(0.0f),
iceCube(false),
scarf(false),
insideNebula(false),
frozen(0.0f),
takenDamage(0.0f),
drawFlames(false)
{
	flames.setPlayer(this);

	keys[Up] = false;
	keys[Down] = false;
	keys[Left] = false;
	keys[Right] = false;
}

//------------------------------------------------------------------------------
PlayerDonkey::~PlayerDonkey()
{

}

//------------------------------------------------------------------------------
void PlayerDonkey::initialise(b2World& world)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2PolygonShape boundingBox;
	b2PolygonShape flamesBox;
	b2PolygonShape chilliBox;

	physicsWorld = &world;

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(64000.0f), pixels2Meters(32000.0f));
	physicsBody = world.CreateBody(&bodyDef);

	boundingBox.SetAsBox(pixels2Meters(72.0f), pixels2Meters(128.0f));
	fixtureDef.shape = &boundingBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);

	//Bottom flames.
	flamesBox.SetAsBox(pixels2Meters(16.0f*4.0f),
					   pixels2Meters(15.0f*4.0f),
					   b2Vec2(pixels2Meters(-7.0f*4.0f),
							  pixels2Meters(42.0f*4.0f)),
					   0.0f);
	fixtureDef.shape = &flamesBox;
	fixtureDef.userData = &flames;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	//Chilli flames.
	chilliBox.SetAsBox(pixels2Meters(12.0f*4.0f),
					   pixels2Meters(15.0f*4.0f),
					   b2Vec2(pixels2Meters(-6.0f*4.0f), -pixels2Meters(40.0f*4.0f)),
					   0.0f);
	fixtureDef.shape = &chilliBox;
	fixtureDef.userData = &chilliFlames;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	physicsBody->SetTransform(bodyDef.position, 1.570796f);
	physicsBody->SetLinearDamping(0.5f);
	physicsBody->SetAngularDamping(0.05f);
}

//------------------------------------------------------------------------------
void PlayerDonkey::move(GamePlay *gamePlay)
{
	float angle = physicsBody->GetAngle();
	const TwoFloats origin;
	TwoFloats tempForce;
	b2Vec2 forceVec;
	float scale;

	if(small)
		scale = 0.25f;
	else
		scale = 1.0f;

	if(keys[Up])
	{
		forceVec.Set(sinf(angle) * 500.0f * speed,
					 -cosf(angle) * 500.0f * speed);
		physicsBody->ApplyForceToCenter(forceVec);
	}
	if(keys[Down])
	{
		//Flip the angle.
		angle += 3.14159265f;

		forceVec.Set(sinf(angle) * 250.0f * speed,
					 -cosf(angle) * 250.0f * speed);
		physicsBody->ApplyForceToCenter(forceVec);
	}
	if(keys[Left])
		physicsBody->ApplyTorque(-1000.0f * speed * scale);
	if(keys[Right])
		physicsBody->ApplyTorque(1000.0f * speed * scale);

	if((teleportPos != origin) && (teleportIndex > 0.5f))
	{
		b2Vec2 newPos;

		newPos.x = pixels2Meters(teleportPos.x);
		newPos.y = pixels2Meters(teleportPos.y);
		physicsBody->SetTransform(newPos, physicsBody->GetAngle());

		teleportPos = origin;
	}

	if(invincible > 0.0f)
		invincible -= 0.002f;

	if(shieldIndex > 0.0f)
	{
		shieldIndex -= 0.002f;
		if(shieldIndex <= 0.0f)
		{
			physicsBody->DestroyFixture(shield);
			shield = 0;
		}
	}

	if(frozen > 0.0f)
	{

		frozen -= 0.005f;

		if(!insideNebula)
		{
			float tempVal;

			tempVal = (1.0f-frozen) * 0.5f;
			tempVal += frozen * 2.0f;
			physicsBody->SetLinearDamping(tempVal);

			tempVal = (1.0f-frozen) * 0.05f;
			tempVal += frozen * 2.0f;
			physicsBody->SetAngularDamping(tempVal);
		}
	}

	if(keys[Up] || drawFlames || fire)
	{
		--sparkCount;
		if(sparkCount <= 0)
		{
			float scale = 4.0f;
			float xScale = 1.0f;
			float angle = physicsBody->GetAngle();
			TwoFloats dir(sinf(angle), -cosf(angle));
			TwoFloats tempPos(getPosition());

			if(small)
				scale = 2.0f;
			if(teleportIndex < 1.0f)
				xScale = 1.0f - sinf(teleportIndex * 3.14159265f);

			tempPos -= (dir*(24.0f*scale)) * xScale;
			tempPos += dir.perpendicular() * ((Random::getGlobal().getRandomFloat()*32.0f)-16.0f);

			gamePlay->addSpark(tempPos, 0.0f-dir);

			sparkCount = 3;
		}
	}
}

//------------------------------------------------------------------------------
void PlayerDonkey::draw(Drawer *d)
{
	ThreeFloats col;
	float scale = 4.0f;
	float xScale = 1.0f;
	b2Vec2 pos = physicsBody->GetPosition();
	const float freezeInterp = fakeLog(1.0f-frozen) * 0.5f;

	if(!image)
	{
		ImageData data;
		ImageLoader imLoad;
		OpenGLDrawer *openGLD = dynamic_cast<OpenGLDrawer *>(d);

		if(openGLD)
			openGLD->setTexInterp(false);

		data = imLoad.loadImage(Images::donkey_png, Images::donkey_pngSize);
		image = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire0_png, Images::bottomfire0_pngSize);
		flameImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire1_png, Images::bottomfire1_pngSize);
		flameImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire2_png, Images::bottomfire2_pngSize);
		flameImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::bottomfire3_png, Images::bottomfire3_pngSize);
		flameImages[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::chillifire0_png, Images::chillifire0_pngSize);
		chilliImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::chillifire1_png, Images::chillifire1_pngSize);
		chilliImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::chillifire2_png, Images::chillifire2_pngSize);
		chilliImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::chillifire3_png, Images::chillifire3_pngSize);
		chilliImages[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::speedyboots_png, Images::speedyboots_pngSize);
		bootsImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::beacon_png, Images::beacon_pngSize);
		beaconImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::circlewave_png, Images::circlewave_pngSize);
		waveImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::unicornhorn_png, Images::unicornhorn_pngSize);
		hornImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::shockwave_png, Images::shockwave_pngSize);
		shieldImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::scarf_png, Images::scarf_pngSize);
		scarfImage = d->loadImage(data.data, data.width, data.height);

		if(openGLD)
			openGLD->setTexInterp(true);
	}

	stars.draw(d, getPosition());

	if(small)
	{
		scale = 2.0f;
		if(updateScale)
			updateFixtures(2.0f);
	}

	if(teleportIndex < 1.0f)
	{
		xScale = 1.0f - sinf(teleportIndex * 3.14159265f);

		teleportIndex += 0.01f;
	}

	if(keys[Up] || drawFlames || fire)
	{
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos -= (dir*(24.0f*scale)) * xScale;
		tempPos += (dir.perpendicular()*16.0f*scale);

		d->drawImageRot(flameImages[flameIndex],
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);

		if(keys[Up] || drawFlames)
		{
			--starsCount;
			if(starsCount <= 0)
			{
				TwoFloats starPos(getPosition());

				starsCount = 5 + (int)(Random::getGlobal().getRandomFloat() * 4.99f);

				starPos -= (dir*(24.0f*4.0f)) * xScale;
				starPos.x += (Random::getGlobal().getRandomFloat() * 32.0f)-16.0f;
				starPos.y += (Random::getGlobal().getRandomFloat() * 32.0f)-16.0f;
				stars.addStar(starPos);
			}
		}
	}
	if(chilli)
	{
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos += (dir*(58.0f*scale)) * xScale;
		tempPos += (dir.perpendicular()*17.0f*scale);

		d->drawImageRot(chilliImages[flameIndex],
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);
	}

	if(!iceCube)
	{
		col.x = 0.5f + freezeInterp;
		col.y = 0.5f + freezeInterp;
		col.z = 1.0f;
	}
	else
	{
		col.x = 0.5f;
		col.y = 0.5f;
		col.z = 1.0f;
	}
	if(takenDamage > 0.0f)
	{
		col *= (1.0f - takenDamage);
		col.x += takenDamage * 1.0f;
		col.y += takenDamage * 0.0f;
		col.z += takenDamage * 0.0f;
	}
	d->drawImageRot(image,
					640.0f-(32.0f*scale) * xScale,
					320.0f-(32.0f*scale)+16.0f,
					physicsBody->GetAngle() * (180.0f/3.14159265f),
					32.0f,
					32.0f,
					scale * xScale,
					scale,
					col.x,
					col.y,
					col.z);

	//Draw boots if we're going fast.
	if(speed > 1.0f)
	{
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos += (dir*(18.0f*scale)) * xScale;
		tempPos -= (dir.perpendicular()*11.0f*scale);

		d->drawImageRot(bootsImage,
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);

		tempPos.x = 640.0f;
		tempPos.y = 320.0f;
		tempPos -= (dir*(10.0f*scale)) * xScale;
		tempPos -= (dir.perpendicular()*10.0f*scale);

		d->drawImageRot(bootsImage,
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);
	}

	//Draw the unicorn horn if we've got it.
	if(invincible > 0.0f)
	{
		float waveScale = beaconIndex;
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos += (dir*(40.0f*scale)) * xScale;
		tempPos += (dir.perpendicular()*20.0f*scale);

		d->drawImageRot(hornImage,
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);
	}

	//Draw the scarf if we've got it.
	if(scarf)
	{
		float waveScale = beaconIndex;
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos += (dir*(28.0f*scale)) * xScale;
		tempPos += (dir.perpendicular()*19.0f*scale);

		d->drawImageRot(scarfImage,
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);
	}

	//Draw the homing beacon if we've got it.
	if(homingBeacon)
	{
		int i;
		float alpha;
		float waveScale = beaconIndex;
		float angle = physicsBody->GetAngle();
		TwoFloats dir(sinf(angle), -cosf(angle));
		TwoFloats tempPos(640.0f, 320.0f);

		tempPos += (dir*(26.0f*scale)) * xScale;
		tempPos += (dir.perpendicular()*24.0f*scale);

		d->drawImageRot(beaconImage,
						tempPos.x,
						tempPos.y+16.0f,
						angle * (180.0f/3.14159265f),
						0.0f,
						0.0f,
						scale * xScale,
						scale);

		tempPos.y += 16.0f;
		for(i=0;i<3;++i)
		{
			waveScale = (beaconIndex/3.0f) + ((float)i/3.0f);
			while(waveScale > 1.0f)
				waveScale -= 1.0f;
			if(waveScale < 0.5f)
				alpha = waveScale * 2.0f;
			else
				alpha = 1.0f-((waveScale-0.5f) * 2.0f);

			waveScale *= 3.0f;
			d->drawImage(waveImage,
						 tempPos.x - (16.0f * (scale*waveScale)),
						 tempPos.y - (16.0f * (scale*waveScale)),
						 scale * waveScale,
						 scale * waveScale,
						 1.0f,
						 1.0f,
						 1.0f,
						 alpha * 0.5f);
		}

		beaconIndex += 0.01f;
		if(beaconIndex > 1.0f)
			beaconIndex -= 1.0f;
	}

	//Draw the shield if we've got it up.
	if(shieldIndex > 0.0f)
	{
		d->drawImage(shieldImage,
					 640.0f - (32.0f * 6.0f),
					 360.0f - (32.0f * 6.0f),
					 6.0f,
					 6.0f,
					 0.75f,
					 0.75f,
					 1.0f,
					 0.5f);
	}

	--flameCount;
	if(flameCount <= 0)
	{
		flameCount = 5;
		++flameIndex;
		flameIndex %= 4;
	}

	if(takenDamage > 0.0f)
		takenDamage -= 0.1f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::keyDown(const std::string& action)
{
	
	if(action == "P1Up")
		keys[Up] = true;
	else if(action == "P1Down")
		keys[Down] = true;
	else if(action == "P1Left")
	{
		keys[Left] = true;
		physicsBody->ApplyAngularImpulse(-500.0f);
	}
	else if(action == "P1Right")
	{
		keys[Right] = true;
		physicsBody->ApplyAngularImpulse(500.0f);
	}
}

//------------------------------------------------------------------------------
void PlayerDonkey::keyUp(const std::string& action)
{
	if(action == "P1Up")
		keys[Up] = false;
	else if(action == "P1Down")
		keys[Down] = false;
	else if(action == "P1Left")
		keys[Left] = false;
	else if(action == "P1Right")
		keys[Right] = false;
}

//------------------------------------------------------------------------------
TwoFloats PlayerDonkey::getPosition()
{
	TwoFloats retval;
	b2Vec2 physicsPos = physicsBody->GetPosition();

	retval.x = meters2Pixels(physicsPos.x);
	retval.y = meters2Pixels(physicsPos.y);

	return retval;
}

//------------------------------------------------------------------------------
void PlayerDonkey::updateLife(float increment)
{
	//Donkey takes less damage if it's wearing a scarf.
	if(scarf && (increment < 0.0f))
		increment *= 0.5f;

	life += increment;
	if(life > 1.0f)
		life = 1.0f;
	else if(life < 0.0f)
		life = 0.0f;

	if(increment < 0.0f)
		takenDamage = 1.0f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::updateSpeed(float increment)
{
	speed += increment;
	if(speed < 0.25f)
		speed = 0.25f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::freeze()
{
	if(!scarf && !iceCube)
	{
		frozen = 1.0f;

		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
}

//------------------------------------------------------------------------------
void PlayerDonkey::gameWin()
{
	drawFlames = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::triggerChilliFlames()
{
	chilli = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setSmall()
{
	small = true;
	updateScale = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setConstantFire()
{
	fire = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setBeacon()
{
	homingBeacon = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::teleport(const TwoFloats& pos)
{
	teleportPos = pos;
	teleportIndex = 0.0f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setInvincible()
{
	invincible = 1.0f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setShield()
{
	if(!shield)
	{
		b2FixtureDef fixtureDef;
		b2CircleShape boundingBox;

		boundingBox.m_p.Set(0.0f, 0.0f);
		boundingBox.m_radius = pixels2Meters(192.0f);
		fixtureDef.shape = &boundingBox;
		fixtureDef.density = 0.1f;
		fixtureDef.friction = 0.3f;
		fixtureDef.userData = this;
		shield = physicsBody->CreateFixture(&fixtureDef);
	}

	shieldIndex = 1.0f;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setIceCube()
{
	iceCube = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::setScarf()
{
	scarf = true;
}

//------------------------------------------------------------------------------
void PlayerDonkey::nebulaCollide(bool val)
{
	insideNebula = val;

	if(insideNebula)
	{
		physicsBody->SetLinearDamping(2.0f);
		physicsBody->SetAngularDamping(2.0f);
	}
	else if(frozen <= 0.0f)
	{
		physicsBody->SetLinearDamping(0.5f);
		physicsBody->SetAngularDamping(0.05f);
	}
}

//------------------------------------------------------------------------------
void PlayerDonkey::updateFixtures(float scale)
{
	b2BodyDef bodyDef;
	b2FixtureDef fixtureDef;
	b2PolygonShape boundingBox;
	b2PolygonShape flamesBox;
	b2PolygonShape chilliBox;
	TwoFloats pos = getPosition();
	float angle = physicsBody->GetAngle();
	b2Vec2 vel = physicsBody->GetLinearVelocity();
	float angVel = physicsBody->GetAngularVelocity();

	assert(scale > 0.0f);

	physicsWorld->DestroyBody(physicsBody);

	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(pixels2Meters(pos.x), pixels2Meters(pos.y));
	physicsBody = physicsWorld->CreateBody(&bodyDef);

	boundingBox.SetAsBox(pixels2Meters(18.0f*scale), pixels2Meters(32.0f*scale));
	fixtureDef.shape = &boundingBox;
	if(small)
		fixtureDef.density = 4.0f;
	else
		fixtureDef.density = 4.0f/scale;
	fixtureDef.friction = 0.3f;
	fixtureDef.userData = this;
	physicsBody->CreateFixture(&fixtureDef);

	//Bottom flames.
	flamesBox.SetAsBox(pixels2Meters(16.0f*scale),
					   pixels2Meters(15.0f*scale),
					   b2Vec2(pixels2Meters(-7.0f*scale),
							  pixels2Meters(42.0f*scale)),
					   0.0f);
	fixtureDef.shape = &flamesBox;
	fixtureDef.userData = &flames;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	//Chilli flames.
	chilliBox.SetAsBox(pixels2Meters(12.0f*scale),
					   pixels2Meters(15.0f*scale),
					   b2Vec2(pixels2Meters(-6.0f*scale), -pixels2Meters(40.0f*scale)),
					   0.0f);
	fixtureDef.shape = &chilliBox;
	fixtureDef.userData = &chilliFlames;
	fixtureDef.isSensor = true;
	physicsBody->CreateFixture(&fixtureDef);

	if(shield)
	{
		b2FixtureDef fixtureDef;
		b2CircleShape boundingBox;

		boundingBox.m_p.Set(0.0f, 0.0f);
		boundingBox.m_radius = pixels2Meters(192.0f);
		fixtureDef.shape = &boundingBox;
		fixtureDef.density = 0.1f;
		fixtureDef.friction = 0.3f;
		fixtureDef.userData = this;
		shield = physicsBody->CreateFixture(&fixtureDef);
	}

	physicsBody->SetTransform(bodyDef.position, angle);
	physicsBody->SetLinearDamping(0.5f);
	physicsBody->SetAngularDamping(0.05f);
	physicsBody->SetLinearVelocity(vel);
	physicsBody->SetAngularVelocity(angVel);

	/*if(scale < 4.0f)
	{
		b2MassData massData;

		physicsBody->GetMassData(&massData);
		massData.mass *= 2.0f;
		physicsBody->SetMassData(&massData);
	}*/
}
