//	PlayerDonkey.h - The player donkey.
//  ----------------------------------------------------------------------------
//	This file is part of 'RainbowSpaceDonkeyEscape', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PLAYERDONKEY_H_
#define PLAYERDONKEY_H_

#include "Collidable.h"
#include "TwoFloats.h"
#include "StarParticles.h"

#include "Box2D/Box2D.h"
#include <stdint.h>
#include <string>

class Drawer;
class GamePlay;
class PlayerDonkey;

//------------------------------------------------------------------------------
///	Simple class representing the donkey's flames for collision purposes.
class DonkeyFlames : public Collidable
{
  public:
	///	Constructor.
	DonkeyFlames(bool chilliFlames = false):
	player(0),
	chilli(chilliFlames)
	{

	};
	///	Destructor.
	~DonkeyFlames() {};

	///	Sets the PlayerDonkey.
	void setPlayer(PlayerDonkey *donkey) {player = donkey;};

	///	Returns true if these are the chilli flames.
	bool getChilli() const {return chilli;};

	///	Used by FlameSpark.
	virtual void setDead() {};
	///	Used by FlameSpark.
	virtual float getLife() const {return 0.0f;};
	///	Used by FlameSpark.
	virtual bool isSpark() const {return false;};

	///	Returns the name.
	const std::string getName() const {return "DonkeyFlames";};
  private:
	///	The associated player.
	PlayerDonkey *player;

	///	True if these are chilli flames.
	bool chilli;
};

//------------------------------------------------------------------------------
///	The player donkey.
class PlayerDonkey : public Collidable
{
  public:
	///	Constructor.
	PlayerDonkey();
	///	Destructor.
	~PlayerDonkey();

	///	Initialises the donkey's physics.
	void initialise(b2World& world);

	///	Moves the donkey.
	void move(GamePlay *gamePlay);
	///	Draws the donkey.
	void draw(Drawer *d);

	///	Handles keypresses.
	void keyDown(const std::string& action);
	///	Handles keypresses.
	void keyUp(const std::string& action);

	///	Returns the player's position.
	TwoFloats getPosition();

	///	Increments/decrements the donkey's life.
	void updateLife(float increment);
	///	Returns the donkey's life (0->1).
	float getLife() const {return life;};

	///	Increments/decrements the donkey's speed.
	void updateSpeed(float increment);
	///	Used to slow the donkey while it's frozen.
	void freeze();

	///	Used to draw the flames after the player's won.
	void gameWin();
	///	Returns true if the player's currently holding the up key.
	bool isFlameOn() const {return (keys[0]||fire);};

	///	Turns the chilli flames on.
	void triggerChilliFlames();
	///	Returns true if the chilli flames are active.
	bool getChilli() const {return chilli;};

	///	Makes the donkey small.
	void setSmall();
	///	Makes the donkey's bottom fire stay on continuously.
	void setConstantFire();
	///	Turns on the donkey's homing beacon.
	void setBeacon();
	///	Teleports the donkey to the position.
	void teleport(const TwoFloats& pos);
	///	Makes the donkey invincible for a short period.
	void setInvincible();
	///	Gives the donkey a shield for a short period.
	void setShield();
	///	So the donkey can freeze santers on touch.
	void setIceCube();
	///	So the donkey takes less damage from santers.
	void setScarf();

	///	Returns true if the donkey's currently invincible.
	bool getInvincible() const {return (invincible > 0.0f);};
	///	Returns true if the donkey's currently got a shield up.
	bool getShield() const {return (shieldIndex > 0.0f);};
	///	Returns true if the donkey should freeze santers on touch.
	bool getIceCube() const {return iceCube;};

	///	Returns the donkey's physics body.
	b2Body *getBody() const {return physicsBody;};

	///	Called when the donkey enters/leaves a nebula.
	void nebulaCollide(bool val);

	///	For collisions.
	const std::string getName() const {return "PlayerDonkey";};
  private:
	///	Helper method. Updates Box2D fixtures based on the scale passed in.
	void updateFixtures(float scale);

	///	The donkey's image.
	static uint32_t image;
	///	The donkey's bottom flame images.
	static uint32_t flameImages[4];
	///	The donkey's chilli flame images.
	static uint32_t chilliImages[4];
	///	The speedy boots image.
	static uint32_t bootsImage;
	///	The homing beacon image.
	static uint32_t beaconImage;
	///	The homing beacon wave image.
	static uint32_t waveImage;
	///	The unicorn horn image.
	static uint32_t hornImage;
	///	The shield wave image.
	static uint32_t shieldImage;
	///	The scarf image.
	static uint32_t scarfImage;

	///	The stars the donkey leaves behind as it flies through space.
	StarParticles stars;
	///	Used to determine when to add a new star particle.
	int starsCount;

	///	The donkey's life.
	float life;

	///	For collision purposes.
	DonkeyFlames flames;
	///	For collision purposes.
	DonkeyFlames chilliFlames;

	///	The Box2D world.
	b2World *physicsWorld;
	///	The donkey's Box2D body.
	b2Body *physicsBody;
	///	For when we've got a sheild up.
	b2Fixture *shield;

	///	Used to move the donkey.
	bool keys[4];

	///	The amount of force the player moves the donkey with.
	float speed;

	///	Used to determine which frame of the fire to display.
	int flameIndex;
	///	Used to determine which frame of the fire to display.
	int flameCount;

	///	Used to determine when to trigger a FlameSpark.
	int sparkCount;

	///	True if the player's got the chilli pickup.
	bool chilli;
	///	True if the player's got the magnifying glass pickup.
	bool small;
	///	So we know we have to update our Box2D body in the next draw call.
	bool updateScale;
	///	True if the player's got the fire pickup.
	bool fire;
	///	True if the player's got the homing beacon pickup.
	bool homingBeacon;
	///	Used to draw the beacon waves.
	float beaconIndex;
	///	Used to teleport the donkey.
	TwoFloats teleportPos;
	///	Used to teleport the donkey.
	float teleportIndex;
	///	> 0 if the donkey's currently invincible.
	float invincible;
	///	> 0 if the donkey's currently got a shield up.
	float shieldIndex;
	///	True if the donkey freezes santers on touch.
	bool iceCube;
	///	True if the donkey takes less damage from santers.
	bool scarf;

	///	True if the donkey's inside a nebula.
	bool insideNebula;
	///	Used to freeze the donkey if it's been hit by an IceSanter.
	float frozen;

	///	Used to flash the donkey red when it's taken damage.
	float takenDamage;

	///	Used to draw the flames after the player's won.
	bool drawFlames;

	///	Names of the keys.
	enum
	{
		Up = 0,
		Down,
		Left,
		Right
	};
};

#endif
